#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# === Mountpoints
#
# This class will scan for all mounted USB-devices first. Then it will
# try to find a specific entry, if you give an argument to .find().
#
# Arguments to find() could be:
#
#   '*'
#
# The above will return an array of all found entries.
#
# We may also store the target device in attr_reader :mounted_here
#
# Additionally, this class may use colours to provide easier to
# read information.
#
# Usage examples:
#   Mountpoints.new
#   Mountpoints.show
#   Mountpoints.is_any_mountpoint_available?
# =========================================================================== #
# require 'mountpoints'
# =========================================================================== #
begin
  require 'colours'
rescue LoadError; end
require 'opn'
require 'mountpoints/version/version.rb'

class Mountpoints # === Mountpoints.new

  include Colours

  N = "\n"

  # ========================================================================= #
  # === LJUST
  # ========================================================================= #
  LJUST = 15

  # ========================================================================= #
  # === DF_COMMAND
  # ========================================================================= #
  DF_COMMAND    = 'df -T -ah'

  # ========================================================================= #
  # === FDISK_COMMAND
  # ========================================================================= #
  FDISK_COMMAND = 'fdisk -l'

  # ========================================================================= #
  # === DEBUG
  # ========================================================================= #
  DEBUG = false # If true then we will debug.

  # ========================================================================= #
  # === initialize
  #
  # If the first argument is :be_extra_verbose then we will show more
  # information.
  # ========================================================================= #
  def initialize(
      run_already = true
    )
    reset
    case run_already
    when :be_extra_verbose
      @be_extra_verbose = true 
      run_already = true
    when :normal_verbosity # Simply pass through here.
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    @be_extra_verbose = false
    @command_to_run = DF_COMMAND # This command will be run.
  end

  # ========================================================================= #
  # === set_result
  # ========================================================================= #
  def set_result(i)
    if i.is_a? String
      i << '/' unless i.end_with? '/'
    elsif i.is_a? Array
      # ===================================================================== #
      # Do not flatten here as we may have more than one Array-entry.
      # ===================================================================== #
      i.map! {|entry|
        entry[0] = rds(entry.first+'/')
        entry
      } # We want directories to end in a '/'.
    end
    if i.is_a? Array
      i.sort_by! {|entry| entry }
    end
    @result = i # Will include trailing /.
    determine_mountpoints
  end

  # ========================================================================= #
  # === rds
  # ========================================================================= #
  def rds(i)
    i.squeeze('/')
  end

  # ========================================================================= #
  # === determine_mountpoints
  #
  # The @array_mountpoints will keep all directory-names with a trailing
  # '/'. In other words, the mountpoints will be stored there.
  # ========================================================================= #
  def determine_mountpoints
    @array_mountpoints = @result.map {|entry| entry.first }
  end

  # ========================================================================= #
  # === determine_result
  #
  # This method will run the df-command.
  # ========================================================================= #
  def determine_result
    if debug?
      e "Now running this command: #{sfancy(@command_to_run)}"
    end
    @result = `#{@command_to_run}`.split(N) # This will keep the main result.
    _ = @result.reject {|entry| !entry.include? '/Mount' }.
                reject {|entry| !entry.include? 'USB' }
    # ======================================================================= #
    # At this point, we have only the entries that have /Mount as part
    # of their name.
    # ======================================================================= #
    _.map! {|entry|
      splitted = entry.split(' ')
      # filesize = 
      entry = [ splitted[-1], splitted.first, splitted[2] ] 
    }
    set_result _ # Now contains the mountpoints here.
  end

  # ========================================================================= #
  # === report_result
  # ========================================================================= #
  def report_result
    if @be_extra_verbose
      if result?.empty?
        opn; e 'We did not find any mount point. We conclude that '\
               'no USB device is mounted.'
      else
        opn; e 'We did find these mount points:'
        e
        result?.each {|line|
          filesize       = line[2]
          name_of_device = line[1]
          mounted_here   = line[0]
          efancy '  '+mounted_here.ljust(LJUST)+'   '+
                 swarn(name_of_device)+' '+
                 '('+Colours.green(filesize)+')'
        }
        e
      end 
    else # else we report "normally.
      e result?.join(N)
    end
  end; alias report_colourized_result report_result # Not sure what to do with this here.

  # ========================================================================= #
  # === result?
  # ========================================================================= #
  def result?
    @result
  end; alias result result? # === result

  # ========================================================================= #
  # === debug?
  # ========================================================================= #
  def debug?
    DEBUG
  end

  # ========================================================================= #
  # === array_mountpoints?
  # ========================================================================= #
  def array_mountpoints? # This should always be an Array.
    @array_mountpoints
  end; alias array_mountpoints array_mountpoints? # === array_mountpoints

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    determine_result
  end

  # ========================================================================= #
  # === Mountpoints.report
  # ========================================================================= #
  def self.report
    Mountpoints[:be_extra_verbose]
  end

  # ========================================================================= #
  # === Mountpoints[]
  #
  # This method will return an array such as:
  #   ["/Mount/USB1/"]
  #
  # In other words, it will be a silent result.
  #
  # Mountpoints[]
  # Mountpoints[:be_extra_verbose]
  # ========================================================================= #
  def self.[](be_extra_verbose = :normal_verbosity)
    if be_extra_verbose == :be_extra_verbose
      _ = Mountpoints.new(be_extra_verbose)
      _.report_result
    else
      Mountpoints.new(be_extra_verbose).array_mountpoints
    end
  end

  # ========================================================================= #
  # === Mountpoints.is_any_mountpoint_available?
  # ========================================================================= #
  def self.is_any_mountpoint_available?
    !Mountpoints.new(:be_quiet).array_mountpoints.empty?
  end

  # ========================================================================= #
  # === Mountpoints.show
  # ========================================================================= #
  def self.show
    Mountpoints.new :be_extra_verbose
  end

end

if __FILE__ == $PROGRAM_NAME
  _ = Mountpoints.new :be_extra_verbose
  _.report_result
  # p _.result?
end # mountpoints